// Page which sets up the connection to Internet and displays a list view of Albums.
// Internet connection is in fact managed by javascript code in HttpRequest.js thanks to
// the XMLHttpRequest object
// A first request is performed to get a session_id. Parsing of the response is done in
// javascript. Then the javascript code sends a second request to get the list of albums.
// Once the response is received, the javascript code returns the response (XML data) in
// xmlData property visible from javascript.
// any change of this property trigs the parsing by the XmlListModel. A XPath syntax (query)
// is used to extract from XML-RPC data (not so simple to find the right syntax!)
// Each item of the list view is displayed by the AlbumDelegate class.
// When an album is clicked, a new page (FilesPage) is pushed on the stack to display files of the
// selected album

import QtQuick 1.1
import com.nokia.meego 1.0

import "Constants.js" as Constants
import "UIConstants.js" as UIConstants
import "HttpRequests.js" as HttpRequests

Page {
    id: albumsPage

    property bool testMode: false
    property string xmlData
    property bool pageBusy: true
    property string spacename
    property string username
    property string password
    property bool initPage : true
    property int sessionType: 0
    property string sessionId
    property string contactId
    property string folderId
    property string label


    // Object to map Xml Response from Joomeo into listview items
    // Only two fields per item are kept to display the list view: label and albumid

    XmlListModel {
        id: xmlAlbumsModel
        xml: xmlData
        query: "/methodResponse/params/param/value/array/data/value/struct"

        XmlRole { name: "folderId"; query: "member[name='folderid']/value/string/string()" }
        XmlRole { name: "albumId"; query: "member[name='albumid']/value/string/string()" }
        XmlRole { name: "label"; query: "member[name='label']/value/string/string()" }
    }



    // Page is made with a header: Rectangle at the top of the screen.
    // Top borders are rounded because display is clipped by the page
    // Two Labels are placed in the Rectangle for the spacename and
    // the count of photos

    Rectangle {
        id: idAlbumsHeader
        width: (screen.currentOrientation == Screen.Portrait) ? 480 : 854
        height: UIConstants.HEADER_HEIGHT

        anchors {
            top: parent.top
        }

        Rectangle {
            color: UIConstants.HEADER_COLOR
            anchors.fill: parent

            Text {
                id: idTextAlbumsHeader
                text: qsTr("Albums of")+" "+spacename
                font.pixelSize: UIConstants.TITLE_FONTSIZE
                anchors {
                    verticalCenter: parent.verticalCenter
                    left:parent.left
                    leftMargin: 10
                }
                color: UIConstants.HEADER_TEXT_COLOR
                maximumLineCount: 1
                elide: Text.ElideRight
            }
            Text {
                id: idTextNbAlbums
                text: (listView.count >= 0 ? listView.count + " "+qsTr("Albums") : "")
                font.pixelSize: UIConstants.TITLE_FONTSIZE
                anchors {
                    verticalCenter: parent.verticalCenter
                    right: parent.right
                    rightMargin: 10
                }
                color: UIConstants.HEADER_TEXT_COLOR
            }
            // Separator
            Rectangle {
                width: parent.width
                height: 1
                anchors.bottom: parent.bottom
                color: UIConstants.SEPARATOR_COLOR
            }
        }
    }

    ListView {
        id: listView
        model: xmlAlbumsModel
        clip: true
        cacheBuffer: 246
        focus: true

        anchors {
            top: idAlbumsHeader.bottom
            left: parent.left
            right: parent.right
            bottom: parent.bottom
        }

        delegate: AlbumsDelegate {
            width: (screen.currentOrientation == Screen.Portrait) ? 480 : 854
            height: UIConstants.ALBUMS_ITEMS_HEIGHT

            session : sessionId

            // Link Xml data format to listview element properties
            label: model.label
            albumId : model.albumId
            folderId : model.folderId

            // Display Files Page when an album is clicked
            onClicked: {

                if(albumId == "")
                {
                    appWindow.pageStack.push(Qt.resolvedUrl("AlbumsPage.qml"),
                                             { sessionId: sessionId,
                                                 spacename: spacename,
                                                 folderId: folderId});
                }
                else
                {
                    appWindow.pageStack.push(Qt.resolvedUrl("FilesPage.qml"),
                                             { sessionId: sessionId,
                                                 albumId: albumId,
                                                 albumLabel: label});
                }
            }
        }
    } // ListView

    ScrollDecorator {
        flickableItem: listView
    }

    // Network error dialog
    ErrorDialog {
        id: networkErrorDialog
        message : qsTr("Network problem\n(Loss of connection ?)")+" "
    } // Dialog


    // Login error dialog
    ErrorDialog {
        id: loginErrorDialog
        message : qsTr("Connection rejected\n(Spacename, Username, password incorrect?)")+" "
    } // Dialog

    // Network timeout timer
    Timer {
        id: timer
        interval: Constants.NETWORK_TIMEOUT
        repeat: false
        onTriggered: HttpRequests.req.abort();
    }

    onStatusChanged: {
        if (status == PageStatus.Active) {
            if(initPage) {
                initPage = false

                if(sessionId == "") {

                    HttpRequests.joomeoSessionInit(spacename, username, password);
                }
                else if (folderId == "" && contactId != "")
                {

                    HttpRequests.joomeoInitContact(sessionId, contactId);
                }
                else
                {

                    HttpRequests.joomeoGetFolderChildren(sessionId, folderId);
                }
            }
        timer.start();
       }
    }


    tools: ToolBarLayout {
        ToolIcon {
            iconId: "toolbar-back"
            onClicked: {
                appWindow.pageStack.pop()
            }
        }

        ToolIcon {
            enabled: false
            width: 180
            Image {
                source: "qrc:///icons/joomeo2.png"
                anchors.centerIn: parent
            }
        }

        ToolIcon {
            id: idToolIconPeople
            iconId: "toolbar-people"
            visible: (sessionType == 0) && (folderId == "") ? true : false
            onClicked: {
                appWindow.pageStack.push(Qt.resolvedUrl("ContactsPage.qml"),
                                         { sessionId: sessionId });
            }
        }

        /* ToolIcon {
            platformIconId: "toolbar-view-menu"
            anchors.right: (parent === undefined) ? undefined : parent.right
            onClicked: (myMenu.status === DialogStatus.Closed) ? myMenu.open() : myMenu.close()
        }*/

    }

    /*Menu {
        id: myMenu
        visualParent: albumsPage
        MenuLayout {
            MenuItem { text: qsTr("Add folder") }
            MenuItem { text: qsTr("Add album") }
        }
    }*/
}
