// Display files (currently only photos) of an albumId
// This page shows either a grid of photos (idGridPhotos), either a grid whose each cell has
// exactly the same size of the screen (idGridLargePhotos). This is the zoom view on each photo.
// The interest is to use the same XML data model for both views (in fact I don't know how to proceed
// with two pages since I know know exactly when pages are destroyed. I'm new to QML dev).
// XML data model is filled with the answer of request made to the server by a javascript function
// defined HttpRequests.js. A XPath expression is used to isolate data in the XML-RPC answer.
// This page does not have a standard toolbar in order to be able to fully display a photo.
// In Grid mode, the toolbar behaves like a standard one. In zoom mode, the toolbar overlays the
// photo with a transparency level. On the top of the screen a details window overlays the photo
// with filename and shooting date. A click on the photo toggles display of the toolbar and the details
// window. A click on the play button of the toolbar, launches a timer to automatically move to the next
// photo
// Two delegates are defined for the grid view and the zoom view. Each delegate contains the id of
// the picture and performs a PHP request to get the picture. The Joomeo API allows to select the
// size of the picture in the PHP request. The grid view requests smaller size than the zoom view.
// To improve:
// - Animation when toolbar is shown (just for fun)
// - Legend, rating to display

import QtQuick 1.1
import QtMobility.systeminfo 1.1
import com.nokia.meego 1.0
import "Constants.js" as Constants
import "UIConstants.js" as UIConstants
import "HttpRequests.js" as HttpRequests

Page {
    id: idFilesPage

    property bool testMode: false
    property string xmlFilesData
    property bool pageBusy: true
    property string sessionId
    property string albumId
    property alias albumLabel : idFilesHeaderText.text
    property bool photoLoaded: false


    XmlListModel {
        id: xmlFilesModel
        xml: xmlFilesData
        query: "/methodResponse/params/param/value/array/data/value/struct"

        XmlRole { name: "fileId"; query: "member[name='fileid']/value/string/string()" }
        XmlRole { name: "fileName"; query: "member[name='filename']/value/string/string()" }
        XmlRole { name: "dateShooting"; query: "member[name='date_shooting']/value/double/string()" }
        XmlRole { name: "fileLegend"; query: "member[name='legend']/value/string/string()" }
    }

    Rectangle {
        id: idFilesHeader
        width: (screen.currentOrientation == Screen.Portrait) ? 480 : 854
        height: UIConstants.HEADER_HEIGHT
        color: UIConstants.HEADER_COLOR

        anchors {
            top: parent.top
            left: parent.left
        }

        Text {
            id: idFilesHeaderText
            color: UIConstants.HEADER_TEXT_COLOR
            font.pixelSize: UIConstants.TITLE_FONTSIZE
            maximumLineCount: 1
            elide: Text.ElideRight

            anchors {
                left: parent.left
                leftMargin: 10
                verticalCenter: parent.verticalCenter
            }
        }

        Text {
            id: idHeaderTextPhotos
            color: UIConstants.HEADER_TEXT_COLOR
            font.pixelSize: UIConstants.TITLE_FONTSIZE
            text: idGridPhotos.count > 0 ? idGridPhotos.count + " " + qsTr("Photos") : ""

            anchors {
                right: parent.right
                rightMargin: 10
                verticalCenter: parent.verticalCenter
            }
        }

        // Separator
        Rectangle {
            width: parent.width
            height: 1
            anchors.bottom: parent.bottom
            color: UIConstants.SEPARATOR_COLOR
        }
    }

    // Grid with photos. Displayed on 3 columns (480/160 = 3) in portrait mode

    GridView {
        id: idGridPhotos

        width: (screen.currentOrientation == Screen.Portrait) ? 480 : 854

        cellHeight: 160
        cellWidth: 160

        clip: true

        anchors {
            top: idFilesHeader.bottom
            left: parent.left
            right: parent.right
            bottom: idToolBar.top
        }

        model: xmlFilesModel

        delegate: FilesDelegate {
            fileId: model.fileId

            onClicked: {
                // When a photo is clicked, the view is changed i.e.
                // grid view is hidden and zoom view is shown
                // toolbar and detailed info are shown with the Play button for slideshow

                idGridPhotos.currentIndex = index
                appWindow.pageStack.push(idZoomedPage)
            }
        }
    }

    // Network timeout timer
    Timer {
        id: timer
        interval: Constants.NETWORK_TIMEOUT
        repeat: false
        onTriggered: HttpRequests.req.abort();
    }

    // When the page is shown, a network request is sent to get
    // the list of files contained in the albumId
    // The javascript code will push the answer into the xmlFilesData string
    // which will trigger the XML parsing and the filling of the GridView objects

    onStatusChanged: {
        if (status == PageStatus.Active) {
            // No need to fill it again if the page is filled
            if(idGridPhotos.count <= 0) {
                HttpRequests.joomeoGetFilesList(sessionId, albumId);
                timer.start();
            }
        }
    }

    // Network error dialog
    ErrorDialog {
        id: networkErrorDialog
        message : qsTr("Network problem\n(Loss of connection ?)")
    } // Dialog


    ToolBar {
        id: idToolBar
        opacity: 0.7

        anchors {
            left: parent.left
            right: parent.right
            bottom: parent.bottom
        }
        tools: ToolBarLayout {

            ToolIcon {
                iconId: "toolbar-back"
                onClicked: {
                    appWindow.pageStack.pop()
                }
            }
            ToolIcon {
                enabled: false
                width: 180
                Image {
                    source: "qrc:///icons/joomeo2.png"
                    anchors.centerIn: parent
                }
            }            ToolIcon {
                id: idToolbarIconPlay
                iconId: "toolbar-mediacontrol-play"
                onClicked: {
                    idGridPhotos.currentIndex = 0
                    pageStack.push(idZoomedPage, {slideShow : true })
                }
            }

            /*
            ToolIcon {
                platformIconId: "toolbar-view-menu"
                anchors.right: (parent === undefined) ? undefined : parent.right
                onClicked: (idFilesPageMenu.status === DialogStatus.Closed) ? idFilesPageMenu.open() : idFilesPageMenu.close()
            }
            */
        }
    }

    /*
    Menu {
        id: idFilesPageMenu
        visualParent: idFilesPage
        MenuLayout {
            MenuItem {
                text: qsTr("Upload files")
                onClicked: {
                    appWindow.pageStack.push(Qt.resolvedUrl("SelectPage.qml"));
                }
            }
            MenuItem { text: qsTr("Rename album") }
            MenuItem { text: qsTr("Remove album") }
        }
    }*/

    Component {

        id: idZoomedPage

        Page {

            property bool slideShow : false

            // This is the zoom view with only one photo per screen.
            // This is a Grid whose each cell are exactly the size of the screen
            // This grid view uses the same XmlListModel as the grid view

            ListView {
                id: idGridLargePhotos

                orientation: ListView.Horizontal

                // Just to be sure that a photo is fully displayed
                snapMode: ListView.SnapToItem

                anchors.fill: parent
                model: xmlFilesModel

                cacheBuffer: 854

                delegate: PhotoDelegate {
                    id: idLargePhoto
                    fileId: model.fileId
                    fileName: model.fileName
                    fileLegend : model.fileLegend
                    dateShooting: model.dateShooting
                    showDetails: idToolBar2.visible

                    // When the screen is touched,
                    // - during a slide show, the slideshow is stopped
                    // - the toolbar and the details window switch between visible/not visible

                    onClicked: {
                        if(idTimerSlideshow.running) {
                            idTimerSlideshow.stop()
                            idToolBar2.visible = true
                        } else
                        {
                            idToolBar2.visible = ! idToolBar2.visible
                        }
                    }

                    // Start Timer if photo was not loaded and has been loaded
                    onImageStatusChanged: {
                        if(ListView.isCurrentItem && idTimerSlideshow.running && imageStatus==Image.Ready) {
                            idTimerSlideshow.restart()
                        }
                    }
                }
            }

            ToolBar {
                id: idToolBar2
                opacity: 0.7

                anchors {
                    left: parent.left
                    right: parent.right
                    bottom: parent.bottom
                }
                tools: ToolBarLayout {

                    ToolIcon {
                        iconId: "toolbar-back"
                        onClicked: {
                            appWindow.pageStack.pop()
                        }
                    }
                    ToolIcon {
                        enabled: false
                        width: 180
                        Image {
                            source: "qrc:///icons/joomeo2.png"
                            anchors.centerIn: parent
                        }
                    }
                    ToolIcon {
                        id: idToolbarIconPlay
                        iconId: "toolbar-mediacontrol-play"
                        onClicked: {
                            idGridPhotos.currentIndex = 0
                            idToolBar2.visible = false
                            idTimerSlideshow.start()
                        }
                    }

                    /*
                    ToolIcon {
                        platformIconId: "toolbar-view-menu"
                        anchors.right: (parent === undefined) ? undefined : parent.right
                        onClicked: (idFilesPageMenu.status === DialogStatus.Closed) ? idFilesPageMenu.open() : idFilesPageMenu.close()
                    }
                    */
                }
            }

            onStatusChanged: {
                if (status == PageStatus.Active) {
                    idGridLargePhotos.positionViewAtIndex(idGridPhotos.currentIndex, ListView.SnapPosition)
                    if(slideShow) {
                        idToolBar2.visible = false
                        idTimerSlideshow.start()
                        slideShow = !slideShow
                    }
                }
            }

            // Slideshow timer
            Timer {
                id: idTimerSlideshow
                interval: 3000
                repeat: true
                onTriggered: {

                    if(idGridLargePhotos.currentItem.imageStatus == Image.Ready) {
                        if (idGridLargePhotos.currentIndex < idGridLargePhotos.count-1)
                            idGridLargePhotos.incrementCurrentIndex()
                        else {
                            idTimerSlideshow.stop()
                            idToolBar2.visible = true
                        }
                    }
                }
            }
        }
    }
}

